@@ -13,8 +13,17 @@ from azure.storage.blob import (
     ContainerClient,
     ContentSettings,
     generate_blob_sas,
+    generate_container_sas,
 )
 
+from ._helpers import (
+    generate_cloud_conn_str,
+    generate_cloud_sas_url,
+    generate_edge_conn_str,
+    generate_edge_sas_url,
+    generate_local_conn_str,
+    generate_local_sas_url,
+)
 from ._types import CredentialType, LocationType
 
 
@@ -25,7 +34,9 @@ class IoTStorageClient:
     location_type: str
     account_name: str
     credential: str
+    connection_string: str
 
+    module: Optional[str] = None
     host: Optional[str] = None
     port: Optional[str] = None
 
@@ -39,6 +50,7 @@ class IoTStorageClient:
         location_type: str,
         account_name: str,
         credential: str,
+        module: Optional[str] = None,
         host: Optional[str] = None,
         port: Optional[str] = None,
     ) -> None:
@@ -46,6 +58,7 @@ class IoTStorageClient:
         self.location_type = location_type
         self.account_name = account_name
         self.credential = credential
+        self.module = module
         self.host = host
         self.port = port
         self.instantiate_service_client()
@@ -57,44 +70,63 @@ class IoTStorageClient:
             f"credential type: {self.credential_type.lower()}\n"
             f"location type: {self.location_type.lower()}\n"
             f"account name: {self.account_name}\n"
-            f"credential: {self.credential[0:5]}****"
+            f"credential: {self.credential[0:3]}****"
         )
 
     def instantiate_service_client(self) -> None:
         """init service_client based on credential and location types"""
         if self.credential_type == CredentialType.CONNECTION_STRING:
+            self.connection_string = self.credential
             self.service_client = BlobServiceClient.from_connection_string(
                 self.credential
             )
-        elif self.credential_type == CredentialType.ACCOUNT_KEY:
-            if self.location_type == LocationType.CLOUD_BASED:
-                self.service_client = BlobServiceClient(
-                    account_url=f"https://{self.account_name}.blob.core.windows.net",
-                    credential=self.credential,
-                )
-            elif self.location_type == LocationType.EDGE_BASED:
-                self.service_client = BlobServiceClient(
-                    account_url=f"http://{self.host}:{self.port}/{self.account_name}.blob.core.windows.net",
-                    credential=self.credential,
-                )
-            else:
-                self.location_type = "INVALID"
-                print(
-                    "invalid location type, please use one of LocationType[CLOUD_BASED, EDGE_BASED]"
-                )
         else:
-            self.credential_type = "INVALID"
-            print(
-                "invalid credential type, please use one of CredentialType[ACCOUNT_KEY, CONNECTION_STRING"
-            )
-
-    def format_account_url(self) -> str:
-        """format the blob account url"""
-        if self.location_type == LocationType.CLOUD_BASED:
-            return f"https://{self.account_name}.blob.core.windows.net"
-        if self.location_type == LocationType.EDGE_BASED:
-            return f"http://{self.host}:{self.port}/{self.account_name}.blob.core.windows.net"
-        return "invalid LocationType"
+            if self.location_type == LocationType.CLOUD_BASED:
+                connection_string = generate_cloud_conn_str(
+                    account=self.account_name,
+                    account_key=self.credential
+                    if self.credential_type == CredentialType.ACCOUNT_KEY
+                    else None,
+                    account_sas=self.credential
+                    if self.credential_type == CredentialType.ACCOUNT_SAS
+                    else None,
+                )
+                self.connection_string = connection_string
+                self.service_client = BlobServiceClient.from_connection_string(
+                    connection_string
+                )
+            if self.location_type == LocationType.EDGE_BASED:
+                connection_string = generate_edge_conn_str(
+                    host=self.host,
+                    port=self.port,
+                    account=self.account_name,
+                    account_key=self.credential
+                    if self.credential_type == CredentialType.ACCOUNT_KEY
+                    else None,
+                    account_sas=self.credential
+                    if self.credential_type == CredentialType.ACCOUNT_SAS
+                    else None,
+                )
+                self.connection_string = connection_string
+                self.service_client = BlobServiceClient.from_connection_string(
+                    connection_string
+                )
+            if self.location_type == LocationType.LOCAL_BASED:
+                connection_string = generate_local_conn_str(
+                    module=self.module,
+                    port=self.port,
+                    account=self.account_name,
+                    account_key=self.credential
+                    if self.credential_type == CredentialType.ACCOUNT_KEY
+                    else None,
+                    account_sas=self.credential
+                    if self.credential_type == CredentialType.ACCOUNT_SAS
+                    else None,
+                )
+                self.connection_string = connection_string
+                self.service_client = BlobServiceClient.from_connection_string(
+                    connection_string
+                )
 
     def container_exists(self, container_name: str) -> bool:
         """check if a container exists"""
@@ -120,6 +152,32 @@ class IoTStorageClient:
             pass
         return False
 
+    def create_container(self, container_name: str) -> bool:
+        """create a new container"""
+        try:
+            container_client = self.service_client.get_container_client(
+                container=container_name
+            )
+            container_client.create_container()
+            return True
+        except Exception as ex:
+            print(f"unexpected exception occurred: {ex}")
+            pass
+        return False
+
+    def delete_container(self, container_name: str) -> bool:
+        """delete a container"""
+        try:
+            container_client = self.service_client.get_container_client(
+                container=container_name
+            )
+            container_client.delete_container()
+            return True
+        except Exception as ex:
+            print(f"unexpected exception occurred: {ex}")
+            pass
+        return False
+
     def download(self, container_name: str, source: str, dest: str) -> bool:
         """download a file or directory to a path on the local filesystem"""
         try:
@@ -139,10 +197,9 @@ class IoTStorageClient:
                 blobs = [source + blob for blob in blobs]
                 for blob in blobs:
                     blob_dest = dest + os.path.relpath(blob, source)
-                    self.download_file(container_name, blob, blob_dest)
+                    return self.download_file(container_name, blob, blob_dest)
             else:
-                self.download_file(container_name, source, dest)
-            return True
+                return self.download_file(container_name, source, dest)
         except Exception as ex:
             print(f"unexpected exception occurred: {ex}")
             pass
@@ -420,10 +477,7 @@ class IoTStorageClient:
     ) -> Union[str, None]:
         """generate a SAS URL for a given file inside the container"""
         try:
-            account_key = self.credential
-            if self.credential_type != CredentialType.ACCOUNT_KEY:
-                account_key = self.service_client.credential.account_key
-
+            account_key = self.service_client.credential.account_key
             sas_token = generate_blob_sas(
                 account_name=self.account_name,
                 container_name=container_name,
@@ -447,8 +501,92 @@ class IoTStorageClient:
                 )
                 return None
 
-            account_url = self.format_account_url()
-            return f"{account_url}/{container_name}/{source}{sas_token}"
+            if self.location_type == LocationType.CLOUD_BASED:
+                return generate_cloud_sas_url(
+                    account=self.account_name,
+                    account_sas=sas_token,
+                    blob_path=f"{container_name}/{source}",
+                )
+            if self.location_type == LocationType.EDGE_BASED:
+                return generate_edge_sas_url(
+                    host=self.host,
+                    port=self.port,
+                    account=self.account_name,
+                    account_sas=sas_token,
+                    blob_path=f"{container_name}/{source}",
+                )
+            if self.location_type == LocationType.LOCAL_BASED:
+                return generate_local_sas_url(
+                    module=self.module,
+                    port=self.port,
+                    account=self.account_name,
+                    account_sas=sas_token,
+                    blob_path=f"{container_name}/{source}",
+                )
+            return None
+        except Exception as ex:
+            print(f"unexpected exception occurred: {ex}")
+            pass
+        return None
+
+    def generate_container_sas_url(
+        self,
+        container_name: str,
+        read: Optional[bool] = True,
+        write: Optional[bool] = False,
+        delete: Optional[bool] = False,
+        start: Optional[Union[datetime, str]] = None,
+        expiry: Optional[Union[datetime, str]] = datetime.utcnow()
+        + timedelta(minutes=15),
+    ) -> Union[str, None]:
+        """generate a SAS URL for a given storage account container"""
+        try:
+            account_key = self.service_client.credential.account_key
+            sas_token = generate_container_sas(
+                account_name=self.account_name,
+                container_name=container_name,
+                account_key=account_key,
+                permission=BlobSasPermissions(
+                    read=read,
+                    add=write,
+                    create=write,
+                    write=write,
+                    delete=delete,
+                    tag=write,
+                ),
+                start=start,
+                expiry=expiry,
+                ip=self.host,
+            )
+            if not sas_token:
+                print(
+                    f"unable to generate SAS token: {self.account_name}/{container_name}"
+                )
+                return None
+
+            if self.location_type == LocationType.CLOUD_BASED:
+                return generate_cloud_sas_url(
+                    account=self.account_name,
+                    account_sas=sas_token,
+                    blob_path=container_name,
+                )
+            if self.location_type == LocationType.EDGE_BASED:
+                return generate_edge_sas_url(
+                    host=self.host,
+                    port=self.port,
+                    account=self.account_name,
+                    account_sas=sas_token,
+                    blob_path=container_name,
+                )
+            if self.location_type == LocationType.LOCAL_BASED:
+                return generate_local_sas_url(
+                    module=self.module,
+                    port=self.port,
+                    account=self.account_name,
+                    account_sas=sas_token,
+                    blob_path=container_name,
+                )
+            return None
         except Exception as ex:
             print(f"unexpected exception occurred: {ex}")
             pass
